/*
   This file contains the main() function.

   You should not modify anything in this file (except
   for one include directive)..

   The main() function launches the thread that maintains
   the "console gui" and then main() goes into an infinite
   loop that creates one customer thread for every input
   character.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <pthread.h>
#include <unistd.h>  // sleep()
#include <curses.h>

typedef struct
{
   char id[4]; // each customer is given an id number
   int state;  // entry, dining, or exit states
   int time;   // how long this customer will be in the dining state
} customer_t;

#define CUSTOMERS 100  // maximum number of customer threads
customer_t* customers[CUSTOMERS];
int customerID = 10;  // initial customer id number


// Four different implementations of the customer thread.
// You need to implement three of these,
// but use only one of them at a time.
#include "customer.h"              // provided by me
//#include "customerBounded.h"     // bound the number of dining customers
//#include "customerSushi.h"       // lock dining area when full
//#include "customerArtGallery.h"  // customers enter dining area as a group

#include "roomDisplay.h"  // this code maintains the "console gui"


int main(int argc, char *argv[])
{
   pthread_t tid;
   int i;
   customer_t* newCustomer;

   (void) initscr();  /* initialize the curses library */
   (void) cbreak();   /* take input chars one at a time, don't wait for \n */
   (void) noecho();   /* don't echo input */

   // initialize the array of customers to NULL pointers
   for (i = 0; i < CUSTOMERS; i++)
      customers[i] = NULL;

   // Create the thread that updates the console window.
   pthread_create(&tid, NULL, roomDisplay, NULL);

   srand(time(NULL));  // seed the random number generator with current time

   while(1) // create one customer thread for each input character
   {
      // read a single input character (in raw mode) from the console
      getch();

      // create a customer_t struct for this thread
      newCustomer = (customer_t*)malloc(sizeof(customer_t));
      // give this customer a number
      //itoa(customerID++, newCustomer->id, 10);
      sprintf(newCustomer->id, "%d", customerID++);
      // give this customer an amount of time for dining
      newCustomer->time = 5+(rand()%20);  // between 5 and 25 seconds
      // put this customer in the entry state
      newCustomer->state = 0;

      // Create a customer thread that will dine for a random time.
      pthread_create(&tid, NULL, customer, (void*)newCustomer);
   }

   return 0;
}//main()
